@@ -18,7 +18,6 @@
 # - ``SERVICE_{TENANT_NAME|PASSWORD}`` must be defined
 # - ``SERVICE_HOST``
 # - ``OS_AUTH_URL``, ``KEYSTONE_SERVICE_URI`` for auth in api
-# - ``CEILOMETER_CONF`` for ceilometer dispatcher configuration
 
 # stack.sh
 # ---------
@@ -39,6 +38,23 @@ set -o xtrace
 GITDIR["python-gnocchiclient"]=$DEST/python-gnocchiclient
 GITREPO["python-gnocchiclient"]=${GNOCCHICLIENT_REPO:-${GIT_BASE}/openstack/python-gnocchiclient.git}
 
+if [ -z "$GNOCCHI_DEPLOY" ]; then
+    # Default
+    GNOCCHI_DEPLOY=werkzeug
+
+    # Fallback to common wsgi devstack configuration
+    if [ "$ENABLE_HTTPD_MOD_WSGI_SERVICES" == "True" ]; then
+        GNOCCHI_DEPLOY=mod_wsgi
+
+    # Deprecated config
+    elif [ -n "$GNOCCHI_USE_MOD_WSGI" ] ; then
+        echo_summary "GNOCCHI_USE_MOD_WSGI is deprecated, use GNOCCHI_DEPLOY instead"
+        if [ "$GNOCCHI_USE_MOD_WSGI" == True ]; then
+            GNOCCHI_DEPLOY=mod_wsgi
+        fi
+    fi
+fi
+
 # Functions
 # ---------
 
@@ -49,6 +65,19 @@ function is_gnocchi_enabled {
     return 1
 }
 
+# Test if a Ceph services are enabled
+# _is_ceph_enabled
+function _is_ceph_enabled {
+    if is_service_enabled ceph; then
+        # Old ceph setup
+        return 0
+    elif type is_ceph_enabled_for_service >/dev/null 2>&1; then
+        # New devstack-plugin-ceph
+        return 0
+    fi
+    return 1
+}
+
 # create_gnocchi_accounts() - Set up common required gnocchi accounts
 
 # Project              User            Roles
@@ -57,25 +86,30 @@ function is_gnocchi_enabled {
 # gnocchi_swift         gnocchi_swift  ResellerAdmin  (if Swift is enabled)
 function create_gnocchi_accounts {
     # Gnocchi
-    if is_service_enabled key && is_service_enabled gnocchi-api
-    then
+    if [ "$GNOCCHI_USE_KEYSTONE" == "True" ] && is_service_enabled gnocchi-api ; then
+        # At this time, the /etc/openstack/clouds.yaml is available,
+        # we could leverage that by setting OS_CLOUD
+        OLD_OS_CLOUD=$OS_CLOUD
+        export OS_CLOUD='devstack-admin'
+
         create_service_user "gnocchi"
 
-        if [[ "$KEYSTONE_CATALOG_BACKEND" = 'sql' ]]; then
-            local gnocchi_service=$(get_or_create_service "gnocchi" \
-                "metric" "OpenStack Metric Service")
-            get_or_create_endpoint $gnocchi_service \
-                "$REGION_NAME" \
-                "$(gnocchi_service_url)/" \
-                "$(gnocchi_service_url)/" \
-                "$(gnocchi_service_url)/"
-        fi
+        local gnocchi_service=$(get_or_create_service "gnocchi" \
+            "metric" "OpenStack Metric Service")
+        get_or_create_endpoint $gnocchi_service \
+            "$REGION_NAME" \
+            "$(gnocchi_service_url)" \
+            "$(gnocchi_service_url)" \
+            "$(gnocchi_service_url)"
+
         if is_service_enabled swift && [[ "$GNOCCHI_STORAGE_BACKEND" = 'swift' ]] ; then
             get_or_create_project "gnocchi_swift" default
             local gnocchi_swift_user=$(get_or_create_user "gnocchi_swift" \
                 "$SERVICE_PASSWORD" default "gnocchi_swift@example.com")
             get_or_add_user_project_role "ResellerAdmin" $gnocchi_swift_user "gnocchi_swift"
         fi
+
+        export OS_CLOUD=$OLD_OS_CLOUD
     fi
 }
 
@@ -115,32 +149,6 @@ function _gnocchi_install_redis {
     pip_install_gr redis
 }
 
-# install influxdb
-# NOTE(chdent): InfluxDB is not currently packaged by the distro at the
-# version that gnocchi needs. Until that is true we're downloading
-# the debs and rpms packaged by the InfluxDB company. When it is
-# true this method can be changed to be similar to
-# _gnocchi_install_redis above.
-function _gnocchi_install_influxdb {
-    if is_package_installed influxdb; then
-        echo "influxdb already installed"
-    else
-        local file=$(mktemp /tmp/influxpkg-XXXXX)
-
-        if is_ubuntu; then
-            wget -O $file $GNOCCHI_INFLUXDB_DEB_PKG
-            sudo dpkg -i $file
-        elif is_fedora; then
-            wget -O $file $GNOCCHI_INFLUXDB_RPM_PKG
-            sudo rpm -i $file
-        fi
-        rm $file
-    fi
-
-    # restart influxdb via its initscript
-    sudo /opt/influxdb/init.sh restart
-}
-
 function _gnocchi_install_grafana {
     if is_ubuntu; then
         local file=$(mktemp /tmp/grafanapkg-XXXXX)
@@ -159,11 +167,6 @@ function _gnocchi_install_grafana {
     sudo service grafana-server restart
 }
 
-# remove the influxdb database
-function _gnocchi_cleanup_influxdb {
-    curl -G 'http://localhost:8086/query' --data-urlencode "q=DROP DATABASE $GNOCCHI_INFLUXDB_DBNAME"
-}
-
 function _cleanup_gnocchi_apache_wsgi {
     sudo rm -f $GNOCCHI_WSGI_DIR/*.wsgi
     sudo rm -f $(apache_site_config_for gnocchi)
@@ -211,7 +214,7 @@ function _config_gnocchi_apache_wsgi {
 # cleanup_gnocchi() - Remove residual data files, anything left over from previous
 # runs that a clean run would need to clean up
 function cleanup_gnocchi {
-    if [ "$GNOCCHI_USE_MOD_WSGI" == "True" ]; then
+    if [ "$GNOCCHI_DEPLOY" == "mod_wsgi" ]; then
         _cleanup_gnocchi_apache_wsgi
     fi
 }
@@ -228,21 +231,35 @@ function configure_gnocchi {
     # Install the configuration files
     cp $GNOCCHI_DIR/etc/gnocchi/* $GNOCCHI_CONF_DIR
 
-    iniset $GNOCCHI_CONF storage coordination_url "$GNOCCHI_COORDINATOR_URL"
-    if [ "${GNOCCHI_COORDINATOR_URL:0:7}" == "file://" ]; then
-        gnocchi_locks_dir=${GNOCCHI_COORDINATOR_URL:7}
-        [ ! -d $gnocchi_locks_dir ] && sudo mkdir -m 755 -p ${gnocchi_locks_dir}
-        sudo chown $STACK_USER $gnocchi_locks_dir
+
+    # Set up logging
+    if [ "$SYSLOG" != "False" ]; then
+        iniset $GNOCCHI_CONF DEFAULT use_syslog "True"
+    fi
+
+    # Format logging
+    if [ "$LOG_COLOR" == "True" ] && [ "$SYSLOG" == "False" ] && [ "$GNOCCHI_DEPLOY" != "mod_wsgi" ]; then
+        setup_colorized_logging $GNOCCHI_CONF DEFAULT
+    fi
+
+    if [ -n "$GNOCCHI_COORDINATOR_URL" ]; then
+        iniset $GNOCCHI_CONF storage coordination_url "$GNOCCHI_COORDINATOR_URL"
     fi
 
     # Configure auth token middleware
     configure_auth_token_middleware $GNOCCHI_CONF gnocchi $GNOCCHI_AUTH_CACHE_DIR
 
+    if is_service_enabled gnocchi-statsd ; then
+        iniset $GNOCCHI_CONF statsd resource_id $GNOCCHI_STATSD_RESOURCE_ID
+        iniset $GNOCCHI_CONF statsd project_id $GNOCCHI_STATSD_PROJECT_ID
+        iniset $GNOCCHI_CONF statsd user_id $GNOCCHI_STATSD_USER_ID
+    fi
+
     # Configure the storage driver
-    if is_service_enabled ceph && [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
+    if _is_ceph_enabled && [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
         iniset $GNOCCHI_CONF storage driver ceph
         iniset $GNOCCHI_CONF storage ceph_username ${GNOCCHI_CEPH_USER}
-        iniset $GNOCCHI_CONF storage ceph_keyring ${CEPH_CONF_DIR}/ceph.client.${GNOCCHI_CEPH_USER}.keyring
+        iniset $GNOCCHI_CONF storage ceph_secret $(awk '/key/{print $3}' ${CEPH_CONF_DIR}/ceph.client.${GNOCCHI_CEPH_USER}.keyring)
     elif is_service_enabled swift && [[ "$GNOCCHI_STORAGE_BACKEND" = 'swift' ]] ; then
         iniset $GNOCCHI_CONF storage driver swift
         iniset $GNOCCHI_CONF storage swift_user gnocchi_swift
@@ -253,31 +270,55 @@ function configure_gnocchi {
     elif [[ "$GNOCCHI_STORAGE_BACKEND" = 'file' ]] ; then
         iniset $GNOCCHI_CONF storage driver file
         iniset $GNOCCHI_CONF storage file_basepath $GNOCCHI_DATA_DIR/
-    elif [[ "$GNOCCHI_STORAGE_BACKEND" == 'influxdb' ]] ; then
-        iniset $GNOCCHI_CONF storage driver influxdb
-        iniset $GNOCCHI_CONF storage influxdb_database $GNOCCHI_INFLUXDB_DBNAME
     else
         echo "ERROR: could not configure storage driver"
         exit 1
     fi
 
-    if is_service_enabled key; then
+    if [ "$GNOCCHI_USE_KEYSTONE" == "True" ] ; then
         if is_service_enabled gnocchi-grafana; then
-            iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline "cors keystone_authtoken gnocchi"
+            iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline "cors gnocchi+auth"
             iniset $KEYSTONE_CONF cors allowed_origin ${GRAFANA_URL}
             iniset $GNOCCHI_CONF cors allowed_origin ${GRAFANA_URL}
             iniset $GNOCCHI_CONF cors allow_methods GET,POST,PUT,DELETE,OPTIONS,HEAD,PATCH
             iniset $GNOCCHI_CONF cors allow_headers Content-Type,Cache-Control,Content-Language,Expires,Last-Modified,Pragma,X-Auth-Token,X-Subject-Token
+        else
+            iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline gnocchi+auth
         fi
     else
-        iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline gnocchi
+        iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline gnocchi+noauth
     fi
 
     # Configure the indexer database
     iniset $GNOCCHI_CONF indexer url `database_connection_url gnocchi`
 
-    if [ "$GNOCCHI_USE_MOD_WSGI" == "True" ]; then
+    if [ "$GNOCCHI_DEPLOY" == "mod_wsgi" ]; then
         _config_gnocchi_apache_wsgi
+    elif [ "$GNOCCHI_DEPLOY" == "uwsgi" ]; then
+        # iniset creates these files when it's called if they don't exist.
+        GNOCCHI_UWSGI_FILE=$GNOCCHI_CONF_DIR/uwsgi.ini
+
+        rm -f "$GNOCCHI_UWSGI_FILE"
+
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi http $GNOCCHI_SERVICE_HOST:$GNOCCHI_SERVICE_PORT
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi wsgi-file "$GNOCCHI_DIR/gnocchi/rest/app.wsgi"
+        # This is running standalone
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi master true
+        # Set die-on-term & exit-on-reload so that uwsgi shuts down
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi die-on-term true
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi exit-on-reload true
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi threads 32
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi processes $API_WORKERS
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi enable-threads true
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi plugins python
+        # uwsgi recommends this to prevent thundering herd on accept.
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi thunder-lock true
+        # Override the default size for headers from the 4k default.
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi buffer-size 65535
+        # Make sure the client doesn't try to re-use the connection.
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi add-header "Connection: close"
+        # Don't share rados resources and python-requests globals between processes
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi lazy-apps true
     fi
 }
 
@@ -290,29 +331,10 @@ function configure_ceph_gnocchi {
         sudo ceph -c ${CEPH_CONF_FILE} osd pool set ${GNOCCHI_CEPH_POOL} crush_ruleset ${RULE_ID}
 
     fi
-    sudo ceph -c ${CEPH_CONF_FILE} auth get-or-create client.${GNOCCHI_CEPH_USER} mon "allow r" osd "allow class-read object_prefix rbd_children, allow rwx pool=${GNOCCHI_CEPH_POOL}" | sudo tee ${CEPH_CONF_DIR}/ceph.client.${GNOCCHI_CEPH_USER}.keyring
+    sudo ceph -c ${CEPH_CONF_FILE} auth get-or-create client.${GNOCCHI_CEPH_USER} mon "allow r" osd "allow rwx pool=${GNOCCHI_CEPH_POOL}" | sudo tee ${CEPH_CONF_DIR}/ceph.client.${GNOCCHI_CEPH_USER}.keyring
     sudo chown ${STACK_USER}:$(id -g -n $whoami) ${CEPH_CONF_DIR}/ceph.client.${GNOCCHI_CEPH_USER}.keyring
 }
 
-function configure_ceilometer_gnocchi {
-    gnocchi_url=$(gnocchi_service_url)
-    iniset $CEILOMETER_CONF DEFAULT dispatcher gnocchi
-    iniset $CEILOMETER_CONF alarms gnocchi_url $gnocchi_url
-    iniset $CEILOMETER_CONF dispatcher_gnocchi url $gnocchi_url
-    iniset $CEILOMETER_CONF dispatcher_gnocchi archive_policy ${GNOCCHI_ARCHIVE_POLICY}
-    if is_service_enabled swift && [[ "$GNOCCHI_STORAGE_BACKEND" = 'swift' ]] ; then
-        iniset $CEILOMETER_CONF dispatcher_gnocchi filter_service_activity "True"
-        iniset $CEILOMETER_CONF dispatcher_gnocchi filter_project "gnocchi_swift"
-    else
-        iniset $CEILOMETER_CONF dispatcher_gnocchi filter_service_activity "False"
-    fi
-}
-
-function configure_aodh_gnocchi {
-    gnocchi_url=$(gnocchi_service_url)
-    iniset $AODH_CONF DEFAULT gnocchi_url $gnocchi_url
-}
-
 
 # init_gnocchi() - Initialize etc.
 function init_gnocchi {
@@ -323,7 +345,24 @@ function init_gnocchi {
 
     if is_service_enabled mysql postgresql; then
         recreate_database gnocchi
-        $GNOCCHI_BIN_DIR/gnocchi-dbsync
+    fi
+    if is_service_enabled ceilometer; then
+        $GNOCCHI_BIN_DIR/gnocchi-upgrade --create-legacy-resource-types
+    else
+        $GNOCCHI_BIN_DIR/gnocchi-upgrade
+    fi
+}
+
+function preinstall_gnocchi {
+    # Needed to build psycopg2
+    if is_ubuntu; then
+        install_package libpq-dev
+    else
+        install_package postgresql-devel
+    fi
+    if [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
+            install_package cython
+            install_package librados-dev
     fi
 }
 
@@ -333,9 +372,8 @@ function install_gnocchi {
         _gnocchi_install_redis
     fi
 
-    if [[ "${GNOCCHI_STORAGE_BACKEND}" == 'influxdb' ]] ; then
-        _gnocchi_install_influxdb
-        pip_install influxdb
+    if [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
+        pip_install cradox
     fi
 
     if is_service_enabled gnocchi-grafana
@@ -345,11 +383,15 @@ function install_gnocchi {
 
     install_gnocchiclient
 
-    # We don't use setup_package because we don't follow openstack/requirements
-    sudo -H pip install -e "$GNOCCHI_DIR"
+    [ "$GNOCCHI_USE_KEYSTONE" == "True" ] && EXTRA_FLAVOR=,keystonmiddleware
 
-    if [ "$GNOCCHI_USE_MOD_WSGI" == "True" ]; then
+    # We don't use setup_package because we don't follow openstack/requirements
+    sudo -H pip install -e "$GNOCCHI_DIR"[test,$GNOCCHI_STORAGE_BACKEND,${DATABASE_TYPE}${EXTRA_FLAVOR}]
+
+    if [ "$GNOCCHI_DEPLOY" == "mod_wsgi" ]; then
         install_apache_wsgi
+    elif [ "$GNOCCHI_DEPLOY" == "uwsgi" ]; then
+        pip_install uwsgi
     fi
 
     # Create configuration directory
@@ -360,7 +402,7 @@ function install_gnocchi {
 # start_gnocchi() - Start running processes, including screen
 function start_gnocchi {
 
-    if [ "$GNOCCHI_USE_MOD_WSGI" == "True" ]; then
+    if [ "$GNOCCHI_DEPLOY" == "mod_wsgi" ]; then
         enable_apache_site gnocchi
         restart_apache_server
         if [[ -n $GNOCCHI_SERVICE_PORT ]]; then
@@ -374,6 +416,8 @@ function start_gnocchi {
             tail_log gnocchi /var/log/$APACHE_NAME/error[_\.]log
             tail_log gnocchi-api /var/log/$APACHE_NAME/access[_\.]log
         fi
+    elif [ "$GNOCCHI_DEPLOY" == "uwsgi" ]; then
+        run_process gnocchi-api "$GNOCCHI_BIN_DIR/uwsgi $GNOCCHI_UWSGI_FILE"
     else
         run_process gnocchi-api "$GNOCCHI_BIN_DIR/gnocchi-api -d -v --config-file $GNOCCHI_CONF"
     fi
@@ -387,27 +431,21 @@ function start_gnocchi {
     fi
 
     # Create a default policy
-    archive_policy_url="$(gnocchi_service_url)/v1/archive_policy"
-    if ! is_service_enabled key; then
+    if [ "$GNOCCHI_USE_KEYSTONE" == "False" ]; then
         export OS_AUTH_TYPE=gnocchi-noauth
         export GNOCCHI_USER_ID=`uuidgen`
         export GNOCCHI_PROJECT_ID=`uuidgen`
-        export GNOCCHI_ENDPOINT="${gnocchi_service_url}"
+        export GNOCCHI_ENDPOINT="$(gnocchi_service_url)"
     fi
 
-    gnocchi archive-policy create -d granularity:5m,points:12 -d granularity:1h,points:24 -d granularity:1d,points:30 low
-    gnocchi archive-policy create -d granularity:60s,points:60 -d granularity:1h,points:168 -d granularity:1d,points:365 medium
-    gnocchi archive-policy create -d granularity:1s,points:86400 -d granularity:1m,points:43200 -d granularity:1h,points:8760 high
-
-    gnocchi archive-policy-rule create -a low -m "*" default
-
     # run metricd last so we are properly waiting for swift and friends
     run_process gnocchi-metricd "$GNOCCHI_BIN_DIR/gnocchi-metricd -d -v --config-file $GNOCCHI_CONF"
+    run_process gnocchi-statsd "$GNOCCHI_BIN_DIR/gnocchi-statsd -d -v --config-file $GNOCCHI_CONF"
 }
 
 # stop_gnocchi() - Stop running processes
 function stop_gnocchi {
-    if [ "$GNOCCHI_USE_MOD_WSGI" == "True" ]; then
+    if [ "$GNOCCHI_DEPLOY" == "mod_wsgi" ]; then
         disable_apache_site gnocchi
         restart_apache_server
     fi
@@ -416,32 +454,23 @@ function stop_gnocchi {
         stop_process $serv
     done
 
-    if [[ "${GNOCCHI_STORAGE_BACKEND}" == 'influxdb' ]] ; then
-        _gnocchi_cleanup_influxdb
-    fi
-
     if is_service_enabled gnocchi-grafana; then
         sudo umount /usr/share/grafana/public/app/plugins/datasource/gnocchi
     fi
 }
 
 if is_service_enabled gnocchi-api; then
-    if [[ "$1" == "stack" && "$2" == "install" ]]; then
+    if [[ "$1" == "stack" && "$2" == "pre-install" ]]; then
+        echo_summary "Configuring system services for Gnocchi"
+        preinstall_gnocchi
+    elif [[ "$1" == "stack" && "$2" == "install" ]]; then
         echo_summary "Installing Gnocchi"
         stack_install_service gnocchi
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         echo_summary "Configuring Gnocchi"
         configure_gnocchi
         create_gnocchi_accounts
-        if is_service_enabled ceilometer; then
-            echo_summary "Configuring Ceilometer for gnocchi"
-            configure_ceilometer_gnocchi
-        fi
-        if is_service_enabled aodh; then
-            echo_summary "Configuring Aodh for gnocchi"
-            configure_aodh_gnocchi
-        fi
-        if is_service_enabled ceph && [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
+        if _is_ceph_enabled && [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
             echo_summary "Configuring Gnocchi for Ceph"
             configure_ceph_gnocchi
         fi
