@@ -22,7 +22,9 @@ import time
 from oslo_config import cfg
 from oslo_utils import timeutils
 import retrying
+import six
 
+from gnocchi import archive_policy
 from gnocchi import indexer
 from gnocchi.rest import app
 from gnocchi import service
@@ -39,19 +41,31 @@ def upgrade():
         cfg.BoolOpt("skip-index", default=False,
                     help="Skip index upgrade."),
         cfg.BoolOpt("skip-storage", default=False,
-                    help="Skip storage upgrade.")
+                    help="Skip storage upgrade."),
+        cfg.BoolOpt("skip-archive-policies-creation", default=False,
+                    help="Skip default archive policies creation."),
+        cfg.BoolOpt("create-legacy-resource-types", default=False,
+                    help="Creation of Ceilometer legacy resource types.")
     ])
     conf = service.prepare_service(conf=conf)
+    index = indexer.get_driver(conf)
+    index.connect()
     if not conf.skip_index:
-        index = indexer.get_driver(conf)
-        index.connect()
         LOG.info("Upgrading indexer %s" % index)
-        index.upgrade()
+        index.upgrade(
+            create_legacy_resource_types=conf.create_legacy_resource_types)
     if not conf.skip_storage:
         s = storage.get_driver(conf)
         LOG.info("Upgrading storage %s" % s)
         s.upgrade(index)
 
+    if (not conf.skip_archive_policies_creation
+            and not index.list_archive_policies()
+            and not index.list_archive_policy_rules()):
+        for name, ap in six.iteritems(archive_policy.DEFAULT_ARCHIVE_POLICIES):
+            index.create_archive_policy(ap)
+        index.create_archive_policy_rule("default", "*", "low")
+
 
 def api():
     app.build_server()
@@ -108,7 +122,8 @@ class MetricReporting(MetricProcessBase):
         try:
             report = self.store.measures_report(details=False)
             if self.queues:
-                block_size = max(16, min(256, len(report) // len(self.queues)))
+                block_size = max(16, min(
+                    256, report['summary']['metrics'] // len(self.queues)))
                 for queue in self.queues:
                     queue.put(block_size)
             LOG.info("Metricd reporting: %d measurements bundles across %d "
@@ -131,6 +146,8 @@ class MetricProcessor(MetricProcessBase):
             if self.queue:
                 while not self.queue.empty():
                     self.block_size = self.queue.get()
+                    LOG.debug("Re-configuring worker to handle up to %s "
+                              "metrics", self.block_size)
             self.store.process_background_tasks(self.index, self.block_size)
         except Exception:
             LOG.error("Unexpected error during measures processing",
@@ -165,7 +182,7 @@ def metricd():
         for worker in workers:
             worker.join()
     except KeyboardInterrupt:
-        _metricd_cleanup(workers, queues)
+        _metricd_cleanup(workers)
         sys.exit(0)
     except Exception:
         LOG.warning("exiting", exc_info=True)
@@ -173,10 +190,10 @@ def metricd():
         sys.exit(1)
 
 
-def _metricd_cleanup(workers, queues):
-    for queue in queues:
-        queue.close()
+def _metricd_cleanup(workers):
     for worker in workers:
+        if hasattr(worker, 'queue'):
+            worker.queue.close()
         worker.terminate()
     for worker in workers:
         worker.join()
