@@ -15,10 +15,11 @@
 # under the License.
 import fnmatch
 import hashlib
+import os
 
+import iso8601
 from oslo_config import cfg
 from oslo_utils import netutils
-import pytz
 import six
 from stevedore import driver
 
@@ -27,7 +28,8 @@ from gnocchi import exceptions
 OPTS = [
     cfg.StrOpt('url',
                secret=True,
-               default="null://",
+               required=True,
+               default=os.getenv("GNOCCHI_INDEXER_URL"),
                help='Indexer driver to use'),
 ]
 
@@ -66,7 +68,7 @@ class Resource(object):
     def lastmodified(self):
         # less precise revision start for Last-Modified http header
         return self.revision_start.replace(microsecond=0,
-                                           tzinfo=pytz.UTC)
+                                           tzinfo=iso8601.iso8601.UTC)
 
 
 def get_driver(conf):
@@ -81,11 +83,11 @@ class IndexerException(Exception):
     """Base class for all exceptions raised by an indexer."""
 
 
-class UnknownResourceType(IndexerException):
+class NoSuchResourceType(IndexerException):
     """Error raised when the resource type is unknown."""
     def __init__(self, type):
-        super(UnknownResourceType, self).__init__(
-            "Resource type %s is unknown" % type)
+        super(NoSuchResourceType, self).__init__(
+            "Resource type %s does not exist" % str(type))
         self.type = type
 
 
@@ -122,6 +124,14 @@ class ArchivePolicyInUse(IndexerException):
         self.archive_policy = archive_policy
 
 
+class ResourceTypeInUse(IndexerException):
+    """Error raised when an resource type is still being used."""
+    def __init__(self, resource_type):
+        super(ResourceTypeInUse, self).__init__(
+            "Resource type %s is still in use" % resource_type)
+        self.resource_type = resource_type
+
+
 class NoSuchArchivePolicyRule(IndexerException):
     """Error raised when an archive policy rule does not exist."""
     def __init__(self, archive_policy_rule):
@@ -156,6 +166,14 @@ class ResourceAlreadyExists(IndexerException):
         self.resource = resource
 
 
+class ResourceTypeAlreadyExists(IndexerException):
+    """Error raised when a resource type already exists."""
+    def __init__(self, resource_type):
+        super(ResourceTypeAlreadyExists, self).__init__(
+            "Resource type %s already exists" % resource_type)
+        self.resource_type = resource_type
+
+
 class ResourceAttributeError(IndexerException, AttributeError):
     """Error raised when an attribute does not exist for a resource type."""
     def __init__(self, resource, attribute):
@@ -236,7 +254,7 @@ class IndexerDriver(object):
         pass
 
     @staticmethod
-    def upgrade(nocreate=False):
+    def upgrade(nocreate=False, create_legacy_resource_types=False):
         pass
 
     @staticmethod
@@ -287,22 +305,14 @@ class IndexerDriver(object):
     def delete_archive_policy_rule(name):
         raise exceptions.NotImplementedError
 
-    @staticmethod
-    def get_metrics(uuids, active_only=True):
-        """Get metrics informations from the indexer.
-
-        :param uuids: A list of metric UUID.
-        :param active_only: Whether to only get active metrics
-        """
-        raise exceptions.NotImplementedError
-
     @staticmethod
     def create_metric(id, created_by_user_id, created_by_project_id,
                       archive_policy_name, name=None, resource_id=None):
         raise exceptions.NotImplementedError
 
     @staticmethod
-    def list_metrics(user_id=None, project_id=None, details=False, **kwargs):
+    def list_metrics(names=None, ids=None, details=False, status='active',
+                     **kwargs):
         raise exceptions.NotImplementedError
 
     @staticmethod
@@ -342,3 +352,30 @@ class IndexerDriver(object):
             if fnmatch.fnmatch(metric_name or "", rule.metric_pattern):
                 return self.get_archive_policy(rule.archive_policy_name)
         raise NoArchivePolicyRuleMatch(metric_name)
+
+    @staticmethod
+    def create_resource_type(resource_type):
+        raise exceptions.NotImplementedError
+
+    @staticmethod
+    def get_resource_type(name):
+        """Get a resource type from the indexer.
+
+        :param name: name of the resource type
+        """
+        raise exceptions.NotImplementedError
+
+    @staticmethod
+    def list_resource_types(attribute_filter=None,
+                            limit=None,
+                            marker=None,
+                            sorts=None):
+        raise exceptions.NotImplementedError
+
+    @staticmethod
+    def get_resource_attributes_schemas():
+        raise exceptions.NotImplementedError
+
+    @staticmethod
+    def get_resource_type_schema():
+        raise exceptions.NotImplementedError
