@@ -32,6 +32,8 @@ import sqlalchemy_utils
 
 from gnocchi import archive_policy
 from gnocchi import indexer
+from gnocchi.indexer import sqlalchemy_legacy_resources as legacy
+from gnocchi import resource_type
 from gnocchi import storage
 from gnocchi import utils
 
@@ -101,7 +103,9 @@ class PreciseTimestamp(types.TypeDecorator):
 
 
 class GnocchiBase(models.ModelBase):
-    pass
+    __table_args__ = (
+        COMMON_TABLES_ARGS,
+    )
 
 
 class ArchivePolicyDefinitionType(sqlalchemy_utils.JSONType):
@@ -119,10 +123,6 @@ class SetType(sqlalchemy_utils.JSONType):
 
 class ArchivePolicy(Base, GnocchiBase, archive_policy.ArchivePolicy):
     __tablename__ = 'archive_policy'
-    __table_args__ = (
-        sqlalchemy.Index('ix_archive_policy_name', 'name'),
-        COMMON_TABLES_ARGS,
-    )
 
     name = sqlalchemy.Column(sqlalchemy.String(255), primary_key=True)
     back_window = sqlalchemy.Column(sqlalchemy.Integer, nullable=False)
@@ -135,7 +135,7 @@ class ArchivePolicy(Base, GnocchiBase, archive_policy.ArchivePolicy):
 class Metric(Base, GnocchiBase, storage.Metric):
     __tablename__ = 'metric'
     __table_args__ = (
-        sqlalchemy.Index('ix_metric_id', 'id'),
+        sqlalchemy.Index('ix_metric_status', 'status'),
         sqlalchemy.UniqueConstraint("resource_id", "name",
                                     name="uniq_metric0resource_id0name"),
         COMMON_TABLES_ARGS,
@@ -152,15 +152,19 @@ class Metric(Base, GnocchiBase, storage.Metric):
         nullable=False)
     archive_policy = sqlalchemy.orm.relationship(ArchivePolicy, lazy="joined")
     created_by_user_id = sqlalchemy.Column(
-        sqlalchemy_utils.UUIDType())
+        sqlalchemy.String(255))
     created_by_project_id = sqlalchemy.Column(
-        sqlalchemy_utils.UUIDType())
+        sqlalchemy.String(255))
     resource_id = sqlalchemy.Column(
         sqlalchemy_utils.UUIDType(),
         sqlalchemy.ForeignKey('resource.id',
-                              ondelete="CASCADE",
+                              ondelete="SET NULL",
                               name="fk_metric_resource_id_resource_id"))
     name = sqlalchemy.Column(sqlalchemy.String(255))
+    status = sqlalchemy.Column(sqlalchemy.Enum('active', 'delete',
+                                               name="metric_status_enum"),
+                               nullable=False,
+                               server_default='active')
 
     def jsonify(self):
         d = {
@@ -197,6 +201,54 @@ class Metric(Base, GnocchiBase, storage.Metric):
     __hash__ = storage.Metric.__hash__
 
 
+RESOURCE_TYPE_SCHEMA_MANAGER = resource_type.ResourceTypeSchemaManager(
+    "gnocchi.indexer.sqlalchemy.resource_type_attribute")
+
+
+def get_legacy_resource_types():
+    resource_types = []
+    for name, attributes in legacy.ceilometer_resources.items():
+        tablename = legacy.ceilometer_tablenames.get(name, name)
+        attrs = RESOURCE_TYPE_SCHEMA_MANAGER.attributes_from_dict(
+            attributes)
+        resource_types.append(ResourceType(name=name,
+                                           tablename=tablename,
+                                           attributes=attrs))
+    return resource_types
+
+
+class ResourceTypeAttributes(sqlalchemy_utils.JSONType):
+    def process_bind_param(self, attributes, dialect):
+        return super(ResourceTypeAttributes, self).process_bind_param(
+            attributes.jsonify(), dialect)
+
+    def process_result_value(self, value, dialect):
+        attributes = super(ResourceTypeAttributes, self).process_result_value(
+            value, dialect)
+        return RESOURCE_TYPE_SCHEMA_MANAGER.attributes_from_dict(attributes)
+
+
+class ResourceType(Base, GnocchiBase, resource_type.ResourceType):
+    __tablename__ = 'resource_type'
+    __table_args__ = (
+        sqlalchemy.UniqueConstraint("tablename",
+                                    name="uniq_resource_type0tablename"),
+        COMMON_TABLES_ARGS,
+    )
+
+    name = sqlalchemy.Column(sqlalchemy.String(255), primary_key=True,
+                             nullable=False)
+    tablename = sqlalchemy.Column(sqlalchemy.String(18), nullable=False)
+    attributes = sqlalchemy.Column(ResourceTypeAttributes)
+
+    def to_baseclass(self):
+        cols = {}
+        for attr in self.attributes:
+            cols[attr.name] = sqlalchemy.Column(attr.satype,
+                                                nullable=not attr.required)
+        return type(str("%s_base" % self.tablename), (object, ), cols)
+
+
 class ResourceJsonifier(indexer.Resource):
     def jsonify(self):
         d = dict(self)
@@ -210,23 +262,24 @@ class ResourceJsonifier(indexer.Resource):
 class ResourceMixin(ResourceJsonifier):
     @declarative.declared_attr
     def __table_args__(cls):
-        return (sqlalchemy.Index('ix_%s_id' % cls.__tablename__, 'id'),
-                sqlalchemy.CheckConstraint('started_at <= ended_at',
+        return (sqlalchemy.CheckConstraint('started_at <= ended_at',
                                            name="ck_started_before_ended"),
                 COMMON_TABLES_ARGS)
 
-    type = sqlalchemy.Column(sqlalchemy.Enum('generic', 'instance',
-                                             'swift_account', 'volume',
-                                             'ceph_account', 'network',
-                                             'identity', 'ipmi', 'stack',
-                                             'image', 'instance_disk',
-                                             'instance_network_interface',
-                                             name="resource_type_enum"),
-                             nullable=False, default='generic')
+    @declarative.declared_attr
+    def type(cls):
+        return sqlalchemy.Column(
+            sqlalchemy.String(255),
+            sqlalchemy.ForeignKey('resource_type.name',
+                                  ondelete="RESTRICT",
+                                  name="fk_%s_resource_type_name" %
+                                  cls.__tablename__),
+            nullable=False)
+
     created_by_user_id = sqlalchemy.Column(
-        sqlalchemy_utils.UUIDType())
+        sqlalchemy.String(255))
     created_by_project_id = sqlalchemy.Column(
-        sqlalchemy_utils.UUIDType())
+        sqlalchemy.String(255))
     started_at = sqlalchemy.Column(PreciseTimestamp, nullable=False,
                                    # NOTE(jd): We would like to use
                                    # sqlalchemy.func.now, but we can't
@@ -238,8 +291,9 @@ class ResourceMixin(ResourceJsonifier):
     revision_start = sqlalchemy.Column(PreciseTimestamp, nullable=False,
                                        default=lambda: utils.utcnow())
     ended_at = sqlalchemy.Column(PreciseTimestamp)
-    user_id = sqlalchemy.Column(sqlalchemy_utils.UUIDType())
-    project_id = sqlalchemy.Column(sqlalchemy_utils.UUIDType())
+    user_id = sqlalchemy.Column(sqlalchemy.String(255))
+    project_id = sqlalchemy.Column(sqlalchemy.String(255))
+    original_resource_id = sqlalchemy.Column(sqlalchemy.String(255))
 
 
 class Resource(ResourceMixin, Base, GnocchiBase):
@@ -249,7 +303,10 @@ class Resource(ResourceMixin, Base, GnocchiBase):
     id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(),
                            primary_key=True)
     revision_end = None
-    metrics = sqlalchemy.orm.relationship(Metric, backref="resource")
+    metrics = sqlalchemy.orm.relationship(
+        Metric, backref="resource",
+        primaryjoin="and_(Resource.id == Metric.resource_id, "
+        "Metric.status == 'active')")
 
     def get_metric(self, metric_name):
         m = super(Resource, self).get_metric(metric_name)
@@ -263,6 +320,7 @@ class Resource(ResourceMixin, Base, GnocchiBase):
 
 class ResourceHistory(ResourceMixin, Base, GnocchiBase):
     __tablename__ = 'resource_history'
+
     revision = sqlalchemy.Column(sqlalchemy.Integer, autoincrement=True,
                                  primary_key=True)
     id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(),
@@ -288,8 +346,7 @@ class ResourceExt(object):
 class ResourceExtMixin(object):
     @declarative.declared_attr
     def __table_args__(cls):
-        return (sqlalchemy.Index('ix_%s_id' % cls.__tablename__, 'id'),
-                COMMON_TABLES_ARGS)
+        return (COMMON_TABLES_ARGS, )
 
     @declarative.declared_attr
     def id(cls):
@@ -298,16 +355,18 @@ class ResourceExtMixin(object):
             sqlalchemy.ForeignKey(
                 'resource.id',
                 ondelete="CASCADE",
-                name="fk_%s_id_resource_id" % cls.__tablename__),
-            primary_key=True)
+                name="fk_%s_id_resource_id" % cls.__tablename__,
+                # NOTE(sileht): We use to ensure that postgresql
+                # does not use AccessExclusiveLock on destination table
+                use_alter=True),
+            primary_key=True
+        )
 
 
 class ResourceHistoryExtMixin(object):
     @declarative.declared_attr
     def __table_args__(cls):
-        return (sqlalchemy.Index('ix_%s_revision' % cls.__tablename__,
-                                 'revision'),
-                COMMON_TABLES_ARGS)
+        return (COMMON_TABLES_ARGS, )
 
     @declarative.declared_attr
     def revision(cls):
@@ -317,16 +376,26 @@ class ResourceHistoryExtMixin(object):
                 'resource_history.revision',
                 ondelete="CASCADE",
                 name="fk_%s_revision_resource_history_revision"
-                % cls.__tablename__),
-            primary_key=True)
+                % cls.__tablename__,
+                # NOTE(sileht): We use to ensure that postgresql
+                # does not use AccessExclusiveLock on destination table
+                use_alter=True),
+            primary_key=True
+        )
+
+
+class HistoryModelIterator(models.ModelIterator):
+    def __next__(self):
+        # NOTE(sileht): Our custom resource attribute columns don't
+        # have the same name in database than in sqlalchemy model
+        # so remove the additional "f_" for the model name
+        n = six.advance_iterator(self.i)
+        model_attr = n[2:] if n[:2] == "f_" else n
+        return model_attr, getattr(self.model, n)
 
 
 class ArchivePolicyRule(Base, GnocchiBase):
     __tablename__ = 'archive_policy_rule'
-    __table_args__ = (
-        sqlalchemy.Index('ix_archive_policy_rule_name', 'name'),
-        COMMON_TABLES_ARGS,
-    )
 
     name = sqlalchemy.Column(sqlalchemy.String(255), primary_key=True)
     archive_policy_name = sqlalchemy.Column(
