@@ -1,5 +1,6 @@
 # -*- encoding: utf-8 -*-
 #
+# Copyright © 2016 Red Hat, Inc.
 # Copyright © 2014-2015 eNovance
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,9 +14,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import itertools
 import uuid
 
-from oslo_log import log
 from oslo_utils import strutils
 import pecan
 from pecan import rest
@@ -30,11 +31,10 @@ from gnocchi import aggregates
 from gnocchi import archive_policy
 from gnocchi import indexer
 from gnocchi import json
+from gnocchi import resource_type
 from gnocchi import storage
 from gnocchi import utils
 
-LOG = log.getLogger(__name__)
-
 
 def arg_to_list(value):
     if isinstance(value, list):
@@ -54,24 +54,8 @@ def abort(status_code, detail='', headers=None, comment=None, **kw):
 
 def get_user_and_project():
     headers = pecan.request.headers
-    # NOTE(jd) If user_id or project_id are UUID, try to convert them in the
-    # proper dashed format. It's indeed possible that a middleware passes
-    # theses UUID without the dash representation. It's valid, we can parse,
-    # but the policy module won't see the equality in the string
-    # representations.
     user_id = headers.get("X-User-Id")
-    if user_id:
-        try:
-            user_id = six.text_type(uuid.UUID(user_id))
-        except Exception:
-            abort(400, "Malformed X-User-Id")
-
     project_id = headers.get("X-Project-Id")
-    if project_id:
-        try:
-            project_id = six.text_type(uuid.UUID(project_id))
-        except Exception:
-            abort(400, "Malformed X-Project-Id")
     return (user_id, project_id)
 
 
@@ -183,9 +167,10 @@ def deserialize_and_validate(schema, required=True):
 
 
 def Timestamp(v):
-    if v is None:
-        return v
-    return utils.to_timestamp(v)
+    t = utils.to_timestamp(v)
+    if t < utils.unix_universal_start:
+        raise ValueError("Timestamp must be after Epoch")
+    return t
 
 
 def PositiveOrNullInt(value):
@@ -242,9 +227,7 @@ def get_pagination_options(params, default):
         sorts = [sorts]
 
     try:
-        limit = int(limit)
-        if limit <= 0:
-            raise ValueError
+        limit = PositiveNotNullInt(limit)
     except ValueError:
         abort(400, "Invalid 'limit' value: %s" % params.get('limit'))
 
@@ -325,7 +308,7 @@ class ArchivePoliciesController(rest.RestController):
         except indexer.ArchivePolicyAlreadyExists as e:
             abort(409, e)
 
-        location = "/v1/archive_policy/" + ap.name
+        location = "/archive_policy/" + ap.name
         set_resp_location_hdr(location)
         pecan.response.status = 201
         return ap
@@ -356,7 +339,7 @@ class ArchivePolicyRulesController(rest.RestController):
         except indexer.ArchivePolicyRuleAlreadyExists as e:
             abort(409, e)
 
-        location = "/v1/archive_policy_rule/" + ap.name
+        location = "/archive_policy_rule/" + ap.name
         set_resp_location_hdr(location)
         pecan.response.status = 201
         return ap
@@ -398,16 +381,18 @@ class AggregatedMetricController(rest.RestController):
 
     @pecan.expose('json')
     def get_measures(self, start=None, stop=None, aggregation='mean',
-                     needed_overlap=100.0):
+                     granularity=None, needed_overlap=100.0):
         return self.get_cross_metric_measures_from_ids(
-            self.metric_ids, start, stop, aggregation, needed_overlap)
+            self.metric_ids, start, stop,
+            aggregation, granularity, needed_overlap)
 
     @classmethod
     def get_cross_metric_measures_from_ids(cls, metric_ids, start=None,
                                            stop=None, aggregation='mean',
+                                           granularity=None,
                                            needed_overlap=100.0):
         # Check RBAC policy
-        metrics = pecan.request.indexer.get_metrics(metric_ids)
+        metrics = pecan.request.indexer.list_metrics(ids=metric_ids)
         missing_metric_ids = (set(metric_ids)
                               - set(six.text_type(m.id) for m in metrics))
         if missing_metric_ids:
@@ -415,11 +400,12 @@ class AggregatedMetricController(rest.RestController):
             abort(404, storage.MetricDoesNotExist(
                 missing_metric_ids.pop()))
         return cls.get_cross_metric_measures_from_objs(
-            metrics, start, stop, aggregation, needed_overlap)
+            metrics, start, stop, aggregation, granularity, needed_overlap)
 
     @staticmethod
     def get_cross_metric_measures_from_objs(metrics, start=None, stop=None,
                                             aggregation='mean',
+                                            granularity=None,
                                             needed_overlap=100.0):
         try:
             needed_overlap = float(needed_overlap)
@@ -449,15 +435,26 @@ class AggregatedMetricController(rest.RestController):
         for metric in metrics:
             enforce("get metric", metric)
 
+        number_of_metrics = len(metrics)
         try:
-            if len(metrics) == 1:
+            if number_of_metrics == 0:
+                return []
+            if granularity is not None:
+                try:
+                    granularity = float(granularity)
+                except ValueError as e:
+                    abort(400, "granularity must be a float: %s" % e)
+            if number_of_metrics == 1:
                 # NOTE(sileht): don't do the aggregation if we only have one
                 # metric
                 measures = pecan.request.storage.get_measures(
-                    metrics[0], start, stop, aggregation)
+                    metrics[0], start, stop, aggregation,
+                    granularity)
             else:
                 measures = pecan.request.storage.get_cross_metric_measures(
-                    metrics, start, stop, aggregation, needed_overlap)
+                    metrics, start, stop, aggregation,
+                    granularity,
+                    needed_overlap)
             # Replace timestamp keys by their string versions
             return [(timestamp.isoformat(), offset, v)
                     for timestamp, offset, v in measures]
@@ -470,6 +467,22 @@ class AggregatedMetricController(rest.RestController):
             abort(404, e)
 
 
+def MeasureSchema(m):
+    # NOTE(sileht): don't use voluptuous for performance reasons
+    try:
+        value = float(m['value'])
+    except Exception:
+        abort(400, "Invalid input for a value")
+
+    try:
+        timestamp = Timestamp(m['timestamp'])
+    except Exception as e:
+        abort(400,
+              "Invalid input for timestamp `%s': %s" % (m['timestamp'], e))
+
+    return storage.Measure(timestamp, value)
+
+
 class MetricController(rest.RestController):
     _custom_actions = {
         'measures': ['POST', 'GET']
@@ -481,23 +494,6 @@ class MetricController(rest.RestController):
                                          invoke_on_load=True)
         self.custom_agg = dict((x.name, x.obj) for x in mgr)
 
-    @staticmethod
-    def to_measure(m):
-        # NOTE(sileht): we do the input validation
-        # during the iteration for not loop just for this
-        # and don't use voluptuous for performance reason
-        try:
-            value = float(m['value'])
-        except Exception:
-            abort(400, "Invalid input for a value")
-
-        try:
-            timestamp = utils.to_timestamp(m['timestamp'])
-        except Exception:
-            abort(400, "Invalid input for a timestamp")
-
-        return storage.Measure(timestamp, value)
-
     def enforce_metric(self, rule):
         enforce(rule, json.to_primitive(self.metric))
 
@@ -514,7 +510,7 @@ class MetricController(rest.RestController):
             abort(400, "Invalid input for measures")
         if params:
             pecan.request.storage.add_measures(
-                self.metric, six.moves.map(self.to_measure, params))
+                self.metric, six.moves.map(MeasureSchema, params))
         pecan.response.status = 202
 
     @pecan.expose('json')
@@ -551,13 +547,13 @@ class MetricController(rest.RestController):
             else:
                 measures = pecan.request.storage.get_measures(
                     self.metric, start, stop, aggregation,
-                    int(granularity) if granularity is not None else None)
+                    float(granularity) if granularity is not None else None)
             # Replace timestamp keys by their string versions
             return [(timestamp.isoformat(), offset, v)
                     for timestamp, offset, v in measures]
-        except storage.MetricDoesNotExist as e:
-            abort(404, e)
-        except storage.AggregationDoesNotExist as e:
+        except (storage.MetricDoesNotExist,
+                storage.GranularityDoesNotExist,
+                storage.AggregationDoesNotExist) as e:
             abort(404, e)
         except aggregates.CustomAggFailure as e:
             abort(400, e)
@@ -571,13 +567,6 @@ class MetricController(rest.RestController):
             abort(404, e)
 
 
-def UUID(value):
-    try:
-        return uuid.UUID(value)
-    except Exception as e:
-        raise ValueError(e)
-
-
 class MetricsController(rest.RestController):
 
     @pecan.expose()
@@ -586,14 +575,15 @@ class MetricsController(rest.RestController):
             metric_id = uuid.UUID(id)
         except ValueError:
             abort(404, indexer.NoSuchMetric(id))
-        metrics = pecan.request.indexer.get_metrics([metric_id])
+        metrics = pecan.request.indexer.list_metrics(
+            id=metric_id, details=True)
         if not metrics:
             abort(404, indexer.NoSuchMetric(id))
         return MetricController(metrics[0]), remainder
 
     _MetricSchema = voluptuous.Schema({
-        "user_id": UUID,
-        "project_id": UUID,
+        "user_id": six.text_type,
+        "project_id": six.text_type,
         "archive_policy_name": six.text_type,
         "name": six.text_type,
     })
@@ -647,7 +637,7 @@ class MetricsController(rest.RestController):
                 archive_policy_name=body['archive_policy_name'])
         except indexer.NoSuchArchivePolicy as e:
             abort(400, e)
-        set_resp_location_hdr("/v1/metric/" + str(m.id))
+        set_resp_location_hdr("/metric/" + str(m.id))
         pecan.response.status = 201
         return m
 
@@ -668,12 +658,16 @@ class MetricsController(rest.RestController):
         else:
             user_id = kwargs.get('user_id')
             project_id = kwargs.get('project_id')
-        return pecan.request.indexer.list_metrics(
-            user_id, project_id)
+        attr_filter = {}
+        if user_id is not None:
+            attr_filter['created_by_user_id'] = user_id
+        if project_id is not None:
+            attr_filter['created_by_project_id'] = project_id
+        return pecan.request.indexer.list_metrics(**attr_filter)
 
 
 _MetricsSchema = voluptuous.Schema({
-    six.text_type: voluptuous.Any(UUID,
+    six.text_type: voluptuous.Any(utils.UUID,
                                   MetricsController.MetricSchema),
 })
 
@@ -697,7 +691,8 @@ class NamedMetricController(rest.RestController):
 
     @pecan.expose()
     def _lookup(self, name, *remainder):
-        m = pecan.request.indexer.list_metrics(details=True,
+        details = True if pecan.request.method == 'GET' else False
+        m = pecan.request.indexer.list_metrics(details=details,
                                                name=name,
                                                resource_id=self.resource_id)
         if m:
@@ -805,25 +800,78 @@ def etag_set_headers(obj):
     pecan.response.last_modified = obj.lastmodified
 
 
+class ResourceTypeController(rest.RestController):
+    def __init__(self, name):
+        self._name = name
+
+    @pecan.expose('json')
+    def get(self):
+        try:
+            rt = pecan.request.indexer.get_resource_type(self._name)
+        except indexer.NoSuchResourceType as e:
+            abort(404, e)
+        enforce("get resource type", rt)
+        return rt
+
+    @pecan.expose()
+    def delete(self):
+        try:
+            pecan.request.indexer.get_resource_type(self._name)
+        except indexer.NoSuchResourceType as e:
+            abort(404, e)
+        enforce("delete resource type", resource_type)
+        try:
+            pecan.request.indexer.delete_resource_type(self._name)
+        except (indexer.NoSuchResourceType,
+                indexer.ResourceTypeInUse) as e:
+            abort(400, e)
+
+
+class ResourceTypesController(rest.RestController):
+
+    @pecan.expose()
+    def _lookup(self, name, *remainder):
+        return ResourceTypeController(name), remainder
+
+    @pecan.expose('json')
+    def post(self):
+        schema = pecan.request.indexer.get_resource_type_schema()
+        body = deserialize_and_validate(schema)
+        rt = schema.resource_type_from_dict(**body)
+        enforce("create resource type", body)
+        try:
+            rt = pecan.request.indexer.create_resource_type(rt)
+        except indexer.ResourceTypeAlreadyExists as e:
+            abort(409, e)
+        set_resp_location_hdr("/resource_type/" + rt.name)
+        pecan.response.status = 201
+        return rt
+
+    @pecan.expose('json')
+    def get_all(self, **kwargs):
+        enforce("list resource type", {})
+        try:
+            return pecan.request.indexer.list_resource_types()
+        except indexer.IndexerException as e:
+            abort(400, e)
+
+
 def ResourceSchema(schema):
     base_schema = {
-        "id": utils.ResourceUUID,
         voluptuous.Optional('started_at'): Timestamp,
         voluptuous.Optional('ended_at'): Timestamp,
-        voluptuous.Optional('user_id'): voluptuous.Any(None, UUID),
-        voluptuous.Optional('project_id'): voluptuous.Any(None, UUID),
+        voluptuous.Optional('user_id'): voluptuous.Any(None, six.text_type),
+        voluptuous.Optional('project_id'): voluptuous.Any(None, six.text_type),
         voluptuous.Optional('metrics'): MetricsSchema,
     }
     base_schema.update(schema)
     return base_schema
 
 
-class GenericResourceController(rest.RestController):
-    _resource_type = 'generic'
+class ResourceController(rest.RestController):
 
-    Resource = ResourceSchema({})
-
-    def __init__(self, id):
+    def __init__(self, resource_type, id):
+        self._resource_type = resource_type
         try:
             self.id = utils.ResourceUUID(id)
         except ValueError:
@@ -852,7 +900,9 @@ class GenericResourceController(rest.RestController):
         enforce("update resource", resource)
         etag_precondition_check(resource)
 
-        body = deserialize_and_validate(self.Resource, required=False)
+        body = deserialize_and_validate(
+            schema_for(self._resource_type),
+            required=False)
 
         if len(body) == 0:
             etag_set_headers(resource)
@@ -870,8 +920,6 @@ class GenericResourceController(rest.RestController):
             create_revision = False
 
         try:
-            if 'metrics' in body:
-                user, project = get_user_and_project()
             resource = pecan.request.indexer.update_resource(
                 self._resource_type,
                 self.id,
@@ -900,89 +948,33 @@ class GenericResourceController(rest.RestController):
             abort(404, e)
 
 
-class SwiftAccountResourceController(GenericResourceController):
-    _resource_type = 'swift_account'
+def schema_for(resource_type):
+    resource_type = pecan.request.indexer.get_resource_type(resource_type)
+    return ResourceSchema(resource_type.schema)
 
 
-class InstanceDiskResourceController(GenericResourceController):
-    _resource_type = 'instance_disk'
-    Resource = ResourceSchema({
-        "name": six.text_type,
-        "instance_id": UUID,
-    })
+def ResourceID(value):
+    return (six.text_type(value), utils.ResourceUUID(value))
 
 
-class InstanceNetworkInterfaceResourceController(GenericResourceController):
-    _resource_type = 'instance_network_interface'
-    Resource = ResourceSchema({
-        "name": six.text_type,
-        "instance_id": UUID,
-    })
-
-
-class InstanceResourceController(GenericResourceController):
-    _resource_type = 'instance'
-
-    Resource = ResourceSchema({
-        "flavor_id": six.text_type,
-        "image_ref": six.text_type,
-        "host": six.text_type,
-        "display_name": six.text_type,
-        voluptuous.Optional("server_group"): six.text_type,
-    })
-
-
-class VolumeResourceController(GenericResourceController):
-    _resource_type = 'volume'
-
-    Resource = ResourceSchema({
-        "display_name": six.text_type,
-    })
-
-
-class CephAccountResourceController(GenericResourceController):
-    _resource_type = 'ceph_account'
-
-
-class NetworkResourceController(GenericResourceController):
-    _resource_type = 'network'
-
-
-class IdentityResourceController(GenericResourceController):
-    _resource_type = 'identity'
-
-
-class IPMIResourceController(GenericResourceController):
-    _resource_type = 'ipmi'
-
-
-class StackResourceController(GenericResourceController):
-    _resource_type = 'stack'
-
-
-class ImageResourceController(GenericResourceController):
-    _resource_type = 'image'
-
-    Resource = ResourceSchema({
-        "name": six.text_type,
-        "container_format": six.text_type,
-        "disk_format": six.text_type,
-    })
-
-
-class GenericResourcesController(rest.RestController):
-    _resource_type = 'generic'
-    _resource_rest_class = GenericResourceController
-
-    Resource = GenericResourceController.Resource
+class ResourcesController(rest.RestController):
+    def __init__(self, resource_type):
+        self._resource_type = resource_type
 
     @pecan.expose()
     def _lookup(self, id, *remainder):
-        return self._resource_rest_class(id), remainder
+        return ResourceController(self._resource_type, id), remainder
 
     @pecan.expose('json')
     def post(self):
-        body = deserialize_and_validate(self.Resource)
+        # NOTE(sileht): we need to copy the dict because when change it
+        # and we don't want that next patch call have the "id"
+        schema = dict(schema_for(self._resource_type))
+        schema["id"] = ResourceID
+
+        body = deserialize_and_validate(schema)
+        body["original_resource_id"], body["id"] = body["id"]
+
         target = {
             "resource_type": self._resource_type,
         }
@@ -1001,7 +993,7 @@ class GenericResourcesController(rest.RestController):
             abort(400, e)
         except indexer.ResourceAlreadyExists as e:
             abort(409, e)
-        set_resp_location_hdr("/v1/resource/"
+        set_resp_location_hdr("/resource/"
                               + self._resource_type + "/"
                               + six.text_type(resource.id))
         etag_set_headers(resource)
@@ -1032,91 +1024,21 @@ class GenericResourcesController(rest.RestController):
             abort(400, e)
 
 
-class SwiftAccountsResourcesController(GenericResourcesController):
-    _resource_type = 'swift_account'
-    _resource_rest_class = SwiftAccountResourceController
-
-
-class InstanceDisksResourcesController(GenericResourcesController):
-    _resource_type = 'instance_disk'
-    _resource_rest_class = InstanceDiskResourceController
-
-    Resource = InstanceDiskResourceController.Resource
-
-
-class InstanceNetworkInterfacesResourcesController(GenericResourcesController):
-    _resource_type = 'instance_network_interface'
-    _resource_rest_class = InstanceNetworkInterfaceResourceController
-
-    Resource = InstanceNetworkInterfaceResourceController.Resource
-
-
-class InstancesResourcesController(GenericResourcesController):
-    _resource_type = 'instance'
-    _resource_rest_class = InstanceResourceController
-
-    Resource = InstanceResourceController.Resource
-
-
-class VolumesResourcesController(GenericResourcesController):
-    _resource_type = 'volume'
-    _resource_rest_class = VolumeResourceController
-
-    Resource = VolumeResourceController.Resource
-
-
-class CephAccountsResourcesController(GenericResourcesController):
-    _resource_type = 'ceph_account'
-    _resource_rest_class = CephAccountResourceController
-
-
-class NetworkResourcesController(GenericResourcesController):
-    _resource_type = 'network'
-    _resource_rest_class = NetworkResourceController
-
-
-class IdentityResourcesController(GenericResourcesController):
-    _resource_type = 'identity'
-    _resource_rest_class = IdentityResourceController
-
-
-class IPMIResourcesController(GenericResourcesController):
-    _resource_type = 'ipmi'
-    _resource_rest_class = IPMIResourceController
-
-
-class StackResourcesController(GenericResourcesController):
-    _resource_type = 'stack'
-    _resource_rest_class = StackResourceController
-
-
-class ImageResourcesController(GenericResourcesController):
-    _resource_type = 'image'
-    _resource_rest_class = ImageResourceController
-
-    Resource = ImageResourceController.Resource
-
-
-class ResourcesController(rest.RestController):
-    resources_ctrl_by_type = dict(
-        (ext.name, ext.plugin())
-        for ext in extension.ExtensionManager(
-            'gnocchi.controller.resources').extensions)
-
+class ResourcesByTypeController(rest.RestController):
     @pecan.expose('json')
     def get_all(self):
         return dict(
-            (type_name,
-             pecan.request.application_url + '/v1/resource/' + type_name)
-            for type_name in self.resources_ctrl_by_type.keys())
+            (rt.name,
+             pecan.request.application_url + '/resource/' + rt.name)
+            for rt in pecan.request.indexer.list_resource_types())
 
     @pecan.expose()
     def _lookup(self, resource_type, *remainder):
-        ctrl = self.resources_ctrl_by_type.get(resource_type)
-        if ctrl:
-            return ctrl, remainder
-        else:
-            abort(404, indexer.UnknownResourceType(resource_type))
+        try:
+            pecan.request.indexer.get_resource_type(resource_type)
+        except indexer.NoSuchResourceType as e:
+            abort(404, e)
+        return ResourcesController(resource_type), remainder
 
 
 def _ResourceSearchSchema(v):
@@ -1151,8 +1073,7 @@ class SearchResourceTypeController(rest.RestController):
         )
     )
 
-    @pecan.expose('json')
-    def post(self, **kwargs):
+    def _search(self, **kwargs):
         if pecan.request.body:
             attr_filter = deserialize_and_validate(self.ResourceSearchSchema)
         else:
@@ -1175,13 +1096,17 @@ class SearchResourceTypeController(rest.RestController):
             else:
                 attr_filter = policy_filter
 
+        return pecan.request.indexer.list_resources(
+            self._resource_type,
+            attribute_filter=attr_filter,
+            details=details,
+            history=history,
+            **pagination_opts)
+
+    @pecan.expose('json')
+    def post(self, **kwargs):
         try:
-            return pecan.request.indexer.list_resources(
-                self._resource_type,
-                attribute_filter=attr_filter,
-                details=details,
-                history=history,
-                **pagination_opts)
+            return self._search(**kwargs)
         except indexer.IndexerException as e:
             abort(400, e)
 
@@ -1189,10 +1114,11 @@ class SearchResourceTypeController(rest.RestController):
 class SearchResourceController(rest.RestController):
     @pecan.expose()
     def _lookup(self, resource_type, *remainder):
-        if resource_type in ResourcesController.resources_ctrl_by_type:
-            return SearchResourceTypeController(resource_type), remainder
-        else:
-            abort(404, indexer.UnknownResourceType(resource_type))
+        try:
+            pecan.request.indexer.get_resource_type(resource_type)
+        except indexer.NoSuchResourceType as e:
+            abort(404, e)
+        return SearchResourceTypeController(resource_type), remainder
 
 
 def _MetricSearchSchema(v):
@@ -1254,7 +1180,8 @@ class SearchMetricController(rest.RestController):
 
     @pecan.expose('json')
     def post(self, metric_id, start=None, stop=None, aggregation='mean'):
-        metrics = pecan.request.indexer.get_metrics(arg_to_list(metric_id))
+        metrics = pecan.request.indexer.list_metrics(
+            ids=arg_to_list(metric_id))
 
         for metric in metrics:
             enforce("search metric", metric)
@@ -1288,47 +1215,156 @@ class SearchMetricController(rest.RestController):
             abort(400, e)
 
 
+class ResourcesMetricsMeasuresBatchController(rest.RestController):
+    MeasuresBatchSchema = voluptuous.Schema(
+        {utils.ResourceUUID: {six.text_type: [MeasureSchema]}}
+    )
+
+    @pecan.expose()
+    def post(self):
+        body = deserialize_and_validate(self.MeasuresBatchSchema)
+
+        known_metrics = []
+        unknown_metrics = []
+        for resource_id in body:
+            names = body[resource_id].keys()
+            metrics = pecan.request.indexer.list_metrics(
+                names=names, resource_id=resource_id)
+
+            if len(names) != len(metrics):
+                known_names = [m.name for m in metrics]
+                unknown_metrics.extend(
+                    ["%s/%s" % (six.text_type(resource_id), m)
+                     for m in names if m not in known_names])
+
+            known_metrics.extend(metrics)
+
+        if unknown_metrics:
+            abort(400, "Unknown metrics: %s" % ", ".join(
+                sorted(unknown_metrics)))
+
+        for metric in known_metrics:
+            enforce("post measures", metric)
+
+        for metric in known_metrics:
+            measures = body[metric.resource_id][metric.name]
+            pecan.request.storage.add_measures(metric, measures)
+
+        pecan.response.status = 202
+
+
+class MetricsMeasuresBatchController(rest.RestController):
+    # NOTE(sileht): we don't allow to mix both formats
+    # to not have to deal with id collision that can
+    # occurs between a metric_id and a resource_id.
+    # Because while json allow duplicate keys in dict payload
+    # only the last key will be retain by json python module to
+    # build the python dict.
+    MeasuresBatchSchema = voluptuous.Schema(
+        {utils.UUID: [MeasureSchema]}
+    )
+
+    @pecan.expose()
+    def post(self):
+        body = deserialize_and_validate(self.MeasuresBatchSchema)
+        metrics = pecan.request.indexer.list_metrics(ids=body.keys())
+
+        if len(metrics) != len(body):
+            missing_metrics = sorted(set(body) - set(m.id for m in metrics))
+            abort(400, "Unknown metrics: %s" % ", ".join(
+                six.moves.map(str, missing_metrics)))
+
+        for metric in metrics:
+            enforce("post measures", metric)
+
+        for metric in metrics:
+            pecan.request.storage.add_measures(metric, body[metric.id])
+
+        pecan.response.status = 202
+
+
 class SearchController(object):
     resource = SearchResourceController()
     metric = SearchMetricController()
 
 
-class AggregationResource(rest.RestController):
+class AggregationResourceController(rest.RestController):
     def __init__(self, resource_type, metric_name):
         self.resource_type = resource_type
         self.metric_name = metric_name
 
     @pecan.expose('json')
     def post(self, start=None, stop=None, aggregation='mean',
-             needed_overlap=100.0):
-        resources = SearchResourceTypeController(self.resource_type).post()
-        metrics = []
-        for r in resources:
-            m = r.get_metric(self.metric_name)
-            if m:
-                metrics.append(m)
-        return AggregatedMetricController.get_cross_metric_measures_from_objs(
-            metrics, start, stop, aggregation, needed_overlap)
+             granularity=None, needed_overlap=100.0,
+             groupby=None):
+        # First, set groupby in the right format: a sorted list of unique
+        # strings.
+        groupby = sorted(set(arg_to_list(groupby)))
+
+        # NOTE(jd) Sort by groupby so we are sure we do not return multiple
+        # groups when using itertools.groupby later.
+        try:
+            resources = SearchResourceTypeController(
+                self.resource_type)._search(sort=groupby)
+        except indexer.InvalidPagination:
+            abort(400, "Invalid groupby attribute")
+        except indexer.IndexerException as e:
+            abort(400, e)
+
+        if resources is None:
+            return []
+
+        if not groupby:
+            metrics = list(filter(None,
+                                  (r.get_metric(self.metric_name)
+                                   for r in resources)))
+            return AggregatedMetricController.get_cross_metric_measures_from_objs(  # noqa
+                metrics, start, stop, aggregation, granularity, needed_overlap)
+
+        def groupper(r):
+            return tuple((attr, r[attr]) for attr in groupby)
+
+        results = []
+        for key, resources in itertools.groupby(resources, groupper):
+            metrics = list(filter(None,
+                                  (r.get_metric(self.metric_name)
+                                   for r in resources)))
+            results.append({
+                "group": dict(key),
+                "measures": AggregatedMetricController.get_cross_metric_measures_from_objs(  # noqa
+                    metrics, start, stop, aggregation,
+                    granularity, needed_overlap)
+            })
+
+        return results
 
 
-class Aggregation(rest.RestController):
+class AggregationController(rest.RestController):
     _custom_actions = {
         'metric': ['GET'],
     }
 
     @pecan.expose()
-    def _lookup(self, object_type, subtype, key, metric_name, *remainder):
-        if object_type == 'resource' and key == 'metric':
-            return AggregationResource(subtype, metric_name), remainder
-        return super(Aggregation, self)._lookup(object_type, subtype, key,
-                                                metric_name, *remainder)
+    def _lookup(self, object_type, resource_type, key, metric_name,
+                *remainder):
+        if object_type != "resource" or key != "metric":
+            # NOTE(sileht): we want the raw 404 message here
+            # so use directly pecan
+            pecan.abort(404)
+        try:
+            pecan.request.indexer.get_resource_type(resource_type)
+        except indexer.NoSuchResourceType as e:
+            abort(404, e)
+        return AggregationResourceController(resource_type,
+                                             metric_name), remainder
 
     @pecan.expose('json')
     def get_metric(self, metric=None, start=None,
                    stop=None, aggregation='mean',
-                   needed_overlap=100.0):
+                   granularity=None, needed_overlap=100.0):
         return AggregatedMetricController.get_cross_metric_measures_from_ids(
-            arg_to_list(metric), start, stop, aggregation, needed_overlap)
+            arg_to_list(metric), start, stop, aggregation,
+            granularity, needed_overlap)
 
 
 class CapabilityController(rest.RestController):
@@ -1346,10 +1382,30 @@ class CapabilityController(rest.RestController):
 class StatusController(rest.RestController):
     @staticmethod
     @pecan.expose('json')
-    def get():
+    def get(details=True):
         enforce("get status", {})
-        report = pecan.request.storage.measures_report()
-        return {"storage": {"measures_to_process": report}}
+        report = pecan.request.storage.measures_report(details)
+        report_dict = {"storage": {"summary": report['summary']}}
+        if 'details' in report:
+            report_dict["storage"]["measures_to_process"] = report['details']
+        return report_dict
+
+
+class MetricsBatchController(object):
+    measures = MetricsMeasuresBatchController()
+
+
+class ResourcesMetricsBatchController(object):
+    measures = ResourcesMetricsMeasuresBatchController()
+
+
+class ResourcesBatchController(object):
+    metrics = ResourcesMetricsBatchController()
+
+
+class BatchController(object):
+    metrics = MetricsBatchController()
+    resources = ResourcesBatchController()
 
 
 class V1Controller(object):
@@ -1360,8 +1416,10 @@ class V1Controller(object):
             "archive_policy": ArchivePoliciesController(),
             "archive_policy_rule": ArchivePolicyRulesController(),
             "metric": MetricsController(),
-            "resource": ResourcesController(),
-            "aggregation": Aggregation(),
+            "batch": BatchController(),
+            "resource": ResourcesByTypeController(),
+            "resource_type": ResourceTypesController(),
+            "aggregation": AggregationController(),
             "capabilities": CapabilityController(),
             "status": StatusController(),
         }
@@ -1374,18 +1432,16 @@ class V1Controller(object):
             "version": "1.0",
             "links": [
                 {"rel": "self",
-                 "href": pecan.request.application_url + "/v1"}
+                 "href": pecan.request.application_url}
             ] + [
                 {"rel": name,
-                 "href": pecan.request.application_url + "/v1/" + name}
+                 "href": pecan.request.application_url + "/" + name}
                 for name in sorted(self.sub_controllers)
             ]
         }
 
 
-class RootController(object):
-    v1 = V1Controller()
-
+class VersionsController(object):
     @staticmethod
     @pecan.expose('json')
     def index():
