@@ -18,6 +18,7 @@ import collections
 import datetime
 import logging
 import multiprocessing
+import operator
 import threading
 import time
 import uuid
@@ -163,8 +164,8 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                 for d in metric.archive_policy.definition:
                     if d.granularity == granularity:
                         return carbonara.AggregatedTimeSerie(
-                            aggregation_method=aggregation,
                             sampling=granularity,
+                            aggregation_method=aggregation,
                             max_size=d.points)
                 raise storage.GranularityDoesNotExist(metric, granularity)
             else:
@@ -194,40 +195,28 @@ class CarbonaraBasedStorage(storage.StorageDriver):
         )
 
         return carbonara.AggregatedTimeSerie.from_timeseries(
-            timeseries,
             sampling=granularity,
+            aggregation_method=aggregation,
+            timeseries=timeseries,
             max_size=points)
 
     def _add_measures(self, aggregation, archive_policy_def,
                       metric, timeserie):
-        with timeutils.StopWatch() as sw:
-            ts = self._get_measures_timeserie(metric, aggregation,
-                                              archive_policy_def.granularity,
-                                              timeserie.first, timeserie.last)
-            LOG.debug("Retrieve measures"
-                      "for %s/%s/%s in %.2fs"
-                      % (metric.id, aggregation, archive_policy_def.
-                         granularity, sw.elapsed()))
+        ts = self._get_measures_timeserie(metric, aggregation,
+                                          archive_policy_def.granularity,
+                                          timeserie.first, timeserie.last)
         ts.update(timeserie)
-        with timeutils.StopWatch() as sw:
-            for key, split in ts.split():
-                self._store_metric_measures(metric, key, aggregation,
-                                            archive_policy_def.granularity,
-                                            split.serialize())
-            LOG.debug("Store measures for %s/%s/%s in %.2fs"
-                      % (metric.id, aggregation,
-                         archive_policy_def.granularity, sw.elapsed()))
+        for key, split in ts.split():
+            self._store_metric_measures(metric, key, aggregation,
+                                        archive_policy_def.granularity,
+                                        split.serialize())
 
         if ts.last and archive_policy_def.timespan:
-            with timeutils.StopWatch() as sw:
-                oldest_point_to_keep = ts.last - datetime.timedelta(
-                    seconds=archive_policy_def.timespan)
-                self._delete_metric_measures_before(
-                    metric, aggregation, archive_policy_def.granularity,
-                    oldest_point_to_keep)
-                LOG.debug("Expire measures for %s/%s/%s in %.2fs"
-                          % (metric.id, aggregation,
-                             archive_policy_def.granularity, sw.elapsed()))
+            oldest_point_to_keep = ts.last - datetime.timedelta(
+                seconds=archive_policy_def.timespan)
+            self._delete_metric_measures_before(
+                metric, aggregation, archive_policy_def.granularity,
+                oldest_point_to_keep)
 
     def add_measures(self, metric, measures):
         self._store_measures(metric, msgpackutils.dumps(
@@ -301,7 +290,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                     break
                 except storage.AggregationDoesNotExist:
                     # This should not happen, but you never know.
-                    LOG.warn(
+                    LOG.warning(
                         "Metric %s does not have an archive "
                         "for aggregation %s, "
                         "no migration can be done" % (metric, agg_method))
@@ -335,8 +324,12 @@ class CarbonaraBasedStorage(storage.StorageDriver):
             # NOTE(jd): We need to lock the metric otherwise we might delete
             # measures that another worker might be processing. Deleting
             # measurement files under its feet is not nice!
-            with self._lock(metric_id)(blocking=sync):
-                self._delete_unprocessed_measures_for_metric_id(metric_id)
+            try:
+                with self._lock(metric_id)(blocking=sync):
+                    self._delete_unprocessed_measures_for_metric_id(metric_id)
+            except coordination.LockAcquireFailed:
+                LOG.debug("Cannot acquire lock for metric %s, postponing"
+                          "unprocessed measures deletion" % metric_id)
         for metric in metrics:
             lock = self._lock(metric.id)
             agg_methods = list(metric.archive_policy.aggregation_methods)
@@ -354,6 +347,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                                       % metric)
                             continue
 
+                        measures = sorted(measures, key=operator.itemgetter(0))
                         try:
                             with timeutils.StopWatch() as sw:
                                 raw_measures = (
@@ -391,10 +385,24 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                                 block_size=mbs,
                                 back_window=metric.archive_policy.back_window)
 
+                        # NOTE(jd) This is Python where you need such
+                        # hack to pass a variable around a closure,
+                        # sorry.
+                        computed_points = {"number": 0}
+
                         def _map_add_measures(bound_timeserie):
+                            # NOTE (gordc): bound_timeserie is entire set of
+                            # unaggregated measures matching largest
+                            # granularity. the following takes only the points
+                            # affected by new measures for specific granularity
+                            tstamp = max(bound_timeserie.first, measures[0][0])
+                            computed_points['number'] = len(bound_timeserie)
                             self._map_in_thread(
                                 self._add_measures,
-                                ((aggregation, d, metric, bound_timeserie)
+                                ((aggregation, d, metric,
+                                  carbonara.TimeSerie(bound_timeserie.ts[
+                                      carbonara.TimeSerie.round_timestamp(
+                                          tstamp, d.granularity * 10e8):]))
                                  for aggregation in agg_methods
                                  for d in metric.archive_policy.definition))
 
@@ -403,10 +411,19 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                                 measures,
                                 before_truncate_callback=_map_add_measures,
                                 ignore_too_old_timestamps=True)
+                            elapsed = sw.elapsed()
+                            number_of_operations = (
+                                len(agg_methods)
+                                * len(metric.archive_policy.definition)
+                            )
+                            speed = ((number_of_operations
+                                     * computed_points['number']) / elapsed)
                             LOG.debug(
                                 "Computed new metric %s with %d new measures "
-                                "in %.2f seconds"
-                                % (metric.id, len(measures), sw.elapsed()))
+                                "in %.2f seconds (%d points/s, %d measures/s)"
+                                % (metric.id, len(measures), elapsed, speed,
+                                   (number_of_operations * len(measures))
+                                   / elapsed))
 
                         self._store_unaggregated_timeserie(metric,
                                                            ts.serialize())
@@ -453,8 +470,8 @@ class CarbonaraBasedStorage(storage.StorageDriver):
             return [(timestamp.replace(tzinfo=iso8601.iso8601.UTC), r, v)
                     for timestamp, r, v
                     in carbonara.AggregatedTimeSerie.aggregated(
-                        tss, from_timestamp, to_timestamp,
-                        aggregation, needed_overlap)]
+                        tss, aggregation, from_timestamp, to_timestamp,
+                        needed_overlap)]
         except carbonara.UnAggregableTimeseries as e:
             raise storage.MetricUnaggregatable(metrics, e.reason)
 
