@@ -43,6 +43,13 @@ def ResourceUUID(value):
         raise ValueError(e)
 
 
+def UUID(value):
+    try:
+        return uuid.UUID(value)
+    except Exception as e:
+        raise ValueError(e)
+
+
 def to_timestamp(v):
     if isinstance(v, datetime.datetime):
         return v
@@ -93,3 +100,10 @@ def utcnow():
 
 def datetime_utc(*args):
     return datetime.datetime(*args, tzinfo=iso8601.iso8601.UTC)
+
+
+unix_universal_start = datetime_utc(1970, 1, 1)
+
+
+def datetime_to_unix(timestamp):
+    return (timestamp - unix_universal_start).total_seconds()
